//
//  CeBitmapView.m
//  BitmapBleueMac
//
//  Created by Renaud Pradenc on 27/08/10.
//  Copyright 2010 Céroce. All rights reserved.
//

#import "CeBitmapView.h"


@implementation CeBitmapView


#define IMAGES_PAR_SECONDE		30

- (id) initWithFrame:(NSRect)frameRect
{
	if(self = [super initWithFrame:frameRect])
	{
		[self _createBitmapWithSize:frameRect.size];
		timer = [NSTimer scheduledTimerWithTimeInterval:1.0/IMAGES_PAR_SECONDE target:self selector:@selector(_timerDidFire:) userInfo:nil repeats:YES];
	}
	
	return self;
}

- (void) dealloc
{
	[timer invalidate];
	[self _releaseBitmap];
	
	[super dealloc];
}

- (void) _timerDidFire:(NSTimer *)theTimer
{
	phaseR += (2.0*M_PI)/(0.8*IMAGES_PAR_SECONDE);	
	phaseG += (2.0*M_PI)/(0.5*IMAGES_PAR_SECONDE);
	phaseB -= (2.0*M_PI)/(0.3*IMAGES_PAR_SECONDE);
	
	[self setNeedsDisplay:YES];
}


- (void) drawRect:(NSRect) dirtyRect
{
	[self _paintBitmap];
	[self _drawBitmapInCurrentContext];
}

- (void) _createBitmapWithSize:(NSSize)size
{
	width = size.width;
	height = size.height;
	
	// Réserver la mémoire pour la bitmap
	bitmap = malloc(width * height * sizeof(PixelRGBx));
	NSAssert(bitmap != NULL, @"malloc n'a pas pu réserver la mémoire pour la bitmap");
	
	// Créer le CGBitmapContext
	CGColorSpaceRef colorspace = CGColorSpaceCreateDeviceRGB();
	NSAssert(colorspace != NULL, @"Impossible de créer le colorspace");
	
	cgContext = CGBitmapContextCreate (
		bitmap,							// data
		width,							// width
		height,							// height,
		8,								// bitsPerComponent
		width*sizeof(PixelRGBx),		// bytesPerRow
		colorspace,						// colorspace
		kCGImageAlphaNoneSkipLast);		// bitmapInfo
	NSAssert(cgContext != NULL, @"Impossible de créer le CGBitmapContext");
	
	CGColorSpaceRelease(colorspace);
}

- (void) _releaseBitmap
{
	CGContextRelease(cgContext);
	free(bitmap);
}

- (void) _paintBitmap
{
	PixelRGBx *pixelPtr = bitmap;	// Pointer sur le début de la bitmap
	
// Largeur des sinusoïdes (en pixels)
#define PERIOD_RED		300.0	
#define PERIOD_GREEN	150.0
#define PERIOD_BLUE		200.0
	
	UInt32	x, y;
	double lumR, lumG, lumB;
	for(y = 0; y < height; y++)
	{
		for(x = 0; x < width; x++)
		{
			lumR = 127.5 * (sin((x/PERIOD_RED)* 2*M_PI + phaseR) + 1.0);
			lumG = 127.5 * (sin((y/PERIOD_GREEN)* 2*M_PI + phaseG) + 1.0);
			lumB = 127.5 * (sin(((x+y)/(2*PERIOD_BLUE))* 2*M_PI + phaseB) + 1.0);
			
			pixelPtr->all = 0x00000000;
			pixelPtr->component.r = (UInt8)lumR;
			pixelPtr->component.g = (UInt8)lumG;
			pixelPtr->component.b = (UInt8)lumB;
			
			pixelPtr++;
		}
	}
}

- (void) _drawBitmapInCurrentContext
{
	CGImageRef image = CGBitmapContextCreateImage(cgContext);
	NSAssert(image != NULL, @"Impossible de créer CGImage < CGBitmapContext");
	
	CGContextRef currentContext = [[NSGraphicsContext currentContext] graphicsPort];
	CGRect rect = CGRectMake(0, 0, width, height);
	CGContextDrawImage(currentContext, rect, image);
	
	CGImageRelease(image);
}

@end
